import { defineStore } from 'pinia'
import { ref, computed } from 'vue'
import type { Order, CreateOrderRequest } from '@/types/order'

export const useOrderStore = defineStore('orders', () => {
  // State
  const orders = ref<Order[]>([])
  const isLoading = ref(false)
  const error = ref<string | null>(null)
  const wsConnection = ref<WebSocket | null>(null)
  const connectionStatus = ref<'disconnected' | 'connecting' | 'connected'>('disconnected')

  // Getters
  const openOrders = computed(() => {
    return orders.value.filter(order => {
      // Hier können wir später Logik für "offene" Bestellungen hinzufügen
      // Momentan zeigen wir alle Bestellungen an
      return true
    })
  })

  const totalOrders = computed(() => orders.value.length)

  // Actions
  const connectWebSocket = () => {
    try {
      connectionStatus.value = 'connecting'
      // WebSocket-Verbindung zum Backend über relative URL
      const protocol = window.location.protocol === 'https:' ? 'wss:' : 'ws:'
      const wsUrl = `${protocol}//${window.location.host}/api/ws`
      wsConnection.value = new WebSocket(wsUrl)
      
      wsConnection.value.onopen = () => {
        console.log('WebSocket verbunden')
        connectionStatus.value = 'connected'
        error.value = null
        // Nach der Verbindung alle Bestellungen anfordern
        requestAllOrders()
      }

      wsConnection.value.onmessage = (event) => {
        try {
          const data = JSON.parse(event.data)
          handleWebSocketMessage(data)
        } catch (e) {
          console.error('Fehler beim Parsen der WebSocket-Nachricht:', e)
        }
      }

      wsConnection.value.onerror = (event) => {
        console.error('WebSocket Fehler:', event)
        connectionStatus.value = 'disconnected'
        error.value = 'WebSocket-Verbindungsfehler'
      }

      wsConnection.value.onclose = () => {
        console.log('WebSocket-Verbindung geschlossen')
        connectionStatus.value = 'disconnected'
        // Automatische Wiederverbindung nach 5 Sekunden
        setTimeout(() => {
          if (connectionStatus.value === 'disconnected') {
            connectWebSocket()
          }
        }, 5000)
      }
    } catch (e) {
      console.error('Fehler beim Verbinden zum WebSocket:', e)
      connectionStatus.value = 'disconnected'
      error.value = 'Verbindungsfehler'
    }
  }

  const requestAllOrders = () => {
    if (wsConnection.value?.readyState === WebSocket.OPEN) {
      wsConnection.value.send(JSON.stringify({ type: 'get_all_orders' }))
      isLoading.value = true
    }
  }

  const handleWebSocketMessage = (data: any) => {
    switch (data.type) {
      case 'all_orders':
        setOrders(data.orders)
        isLoading.value = false
        break
      case 'order_created':
        addOrder(data.order)
        break
      case 'order_updated':
        updateOrder(data.order)
        break
      case 'order_deleted':
        removeOrder(data.order_id)
        break
      case 'order_created_response':
        handleOrderCreatedResponse(data)
        break
      case 'order_deleted_response':
        handleOrderDeletedResponse(data)
        break
      default:
        console.log('Unbekannte WebSocket-Nachricht:', data)
    }
  }

  const handleOrderCreatedResponse = (data: any) => {
    if (data.success) {
      // Bestellung wurde erfolgreich erstellt
      console.log('Bestellung erfolgreich erstellt:', data.order)
      // Die Bestellung wird bereits über den 'order_created' Broadcast hinzugefügt
    } else {
      // Fehler beim Erstellen der Bestellung
      console.error('Fehler beim Erstellen der Bestellung:', data.error)
      error.value = data.error || 'Unbekannter Fehler beim Erstellen der Bestellung'
    }
  }

  const handleOrderDeletedResponse = (data: any) => {
    if (data.success) {
      // Bestellung wurde erfolgreich gelöscht
      console.log('Bestellung erfolgreich gelöscht:', data.message)
      // Die Bestellung wird bereits über den 'order_deleted' Broadcast entfernt
    } else {
      // Fehler beim Löschen der Bestellung
      console.error('Fehler beim Löschen der Bestellung:', data.error)
      error.value = data.error || 'Unbekannter Fehler beim Löschen der Bestellung'
    }
  }

  const createOrder = async (orderRequest: CreateOrderRequest) => {
    return new Promise((resolve, reject) => {
      if (wsConnection.value?.readyState === WebSocket.OPEN) {
        // WebSocket-Nachricht senden
        wsConnection.value.send(JSON.stringify({
          type: 'create_order',
          drinks: orderRequest.drinks
        }))
        
        // Timeout für die Antwort setzen
        const timeout = setTimeout(() => {
          reject(new Error('Timeout beim Erstellen der Bestellung'))
        }, 10000)
        
        // Temporärer Event-Listener für die Antwort
        const handleResponse = (event: MessageEvent) => {
          try {
            const data = JSON.parse(event.data)
            if (data.type === 'order_created_response') {
              clearTimeout(timeout)
              wsConnection.value?.removeEventListener('message', handleResponse)
              
              if (data.success) {
                resolve(data.order)
              } else {
                reject(new Error(data.error || 'Unbekannter Fehler'))
              }
            }
          } catch (e) {
            // Ignoriere andere Nachrichten
          }
        }
        
        wsConnection.value.addEventListener('message', handleResponse)
      } else {
        // Fallback auf HTTP-API
        createOrderHttp(orderRequest).then(resolve).catch(reject)
      }
    })
  }

  const createOrderHttp = async (orderRequest: CreateOrderRequest) => {
    try {
      const response = await fetch('/api/orders', {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
        },
        body: JSON.stringify(orderRequest),
      })
      
      if (!response.ok) {
        throw new Error(`HTTP error! status: ${response.status}`)
      }
      
      const newOrder = await response.json()
      addOrder(newOrder)
      return newOrder
    } catch (e) {
      console.error('Fehler beim Erstellen der Bestellung:', e)
      error.value = 'Fehler beim Erstellen der Bestellung'
      throw e
    }
  }

  const deleteOrder = async (orderId: string) => {
    return new Promise((resolve, reject) => {
      if (wsConnection.value?.readyState === WebSocket.OPEN) {
        // WebSocket-Nachricht senden
        wsConnection.value.send(JSON.stringify({
          type: 'delete_order',
          order_id: orderId
        }))
        
        // Timeout für die Antwort setzen
        const timeout = setTimeout(() => {
          reject(new Error('Timeout beim Löschen der Bestellung'))
        }, 10000)
        
        // Temporärer Event-Listener für die Antwort
        const handleResponse = (event: MessageEvent) => {
          try {
            const data = JSON.parse(event.data)
            if (data.type === 'order_deleted_response') {
              clearTimeout(timeout)
              wsConnection.value?.removeEventListener('message', handleResponse)
              
              if (data.success) {
                resolve(data.message)
              } else {
                reject(new Error(data.error || 'Unbekannter Fehler'))
              }
            }
          } catch (e) {
            // Ignoriere andere Nachrichten
          }
        }
        
        wsConnection.value.addEventListener('message', handleResponse)
      } else {
        // Fallback auf HTTP-API
        deleteOrderHttp(orderId).then(resolve).catch(reject)
      }
    })
  }

  const deleteOrderHttp = async (orderId: string) => {
    try {
      const response = await fetch(`/api/orders/${orderId}`, {
        method: 'DELETE',
      })
      
      if (!response.ok) {
        throw new Error(`HTTP error! status: ${response.status}`)
      }
      
      const result = await response.json()
      return result.message
    } catch (e) {
      console.error('Fehler beim Löschen der Bestellung:', e)
      error.value = 'Fehler beim Löschen der Bestellung'
      throw e
    }
  }

  const addOrder = (order: Order) => {
    orders.value.unshift(order) // Neue Bestellung am Anfang hinzufügen
  }

  const updateOrder = (updatedOrder: Order) => {
    const index = orders.value.findIndex(order => order.id === updatedOrder.id)
    if (index !== -1) {
      orders.value[index] = updatedOrder
    }
  }

  const removeOrder = (orderId: string) => {
    const index = orders.value.findIndex(order => order.id === orderId)
    if (index !== -1) {
      orders.value.splice(index, 1)
    }
  }

  const setOrders = (newOrders: Order[]) => {
    orders.value = newOrders
  }

  const disconnectWebSocket = () => {
    if (wsConnection.value) {
      wsConnection.value.close()
      wsConnection.value = null
    }
    connectionStatus.value = 'disconnected'
  }

  return {
    // State
    orders,
    isLoading,
    error,
    wsConnection,
    connectionStatus,
    
    // Getters
    openOrders,
    totalOrders,
    
    // Actions
    connectWebSocket,
    disconnectWebSocket,
    requestAllOrders,
    createOrder,
    deleteOrder,
    addOrder,
    updateOrder,
    removeOrder,
    setOrders
  }
}) 